@@ -148,9 +148,12 @@ CREATE TABLE QuotaGroup (
 	phyNumHard	INT8	NOT NULL
 );
 
+
+-- QuotaDirSet doesn't have "REFERENCES GfarmUser(username) ON DELETE CASCADE",
+-- because we preserve the dirset even its owner is removed.
+
 CREATE TABLE QuotaDirSet (
-	username	VARCHAR(64)	NOT NULL
-		REFERENCES GfarmUser(username) ON DELETE CASCADE,
+	username	VARCHAR(64)	NOT NULL,
 	dirSetName	VARCHAR(64)	NOT NULL,
 	gracePeriod	INT8	NOT NULL,
 	fileSpace	INT8	NOT NULL,
