@@ -10,6 +10,10 @@
 #include <libgen.h>
 #include <assert.h>
 
+#if (defined(__hurd__) || defined(__gnu_hurd__)) && !defined(PATH_MAX)
+#define	PATH_MAX	2048	/* XXX FIXME */
+#endif
+
 #include <gfarm/gfarm.h>
 
 #include "config.h"
