@@ -37,7 +37,7 @@ gfarm_server_config_read(void)
 			config_file);
 		return (GFARM_ERRMSG_CANNOT_OPEN_CONFIG);
 	}
-	e = gfarm_config_read_file(config, &lineno);
+	e = gfarm_config_read_file(config, &lineno, config_file);
 	if (e != GFARM_ERR_NO_ERROR) {
 		gflog_error(GFARM_MSG_1000014, "%s: line %d: %s",
 		    config_file, lineno, gfarm_error_string(e));
