@@ -8,6 +8,8 @@
  * Every other modules shouldn't include this.
  */
 
+#include <pthread.h>
+
 struct stat;
 
 #define	GFS_FILE_IS_PROGRAM(gf) (GFARM_S_IS_PROGRAM(gf->pi.status.st_mode))
@@ -75,6 +77,7 @@ struct gfs_file {
  * GFS_PROTO_MAX_IOSIZE defined in gfs_proto.h.
  */
 	char *buffer;
+
 	int bufsize;
 	int p;
 	int length;
@@ -106,6 +109,8 @@ struct gfs_file {
 
 	/* opening files */
 	GFARM_HCIRCLEQ_ENTRY(gfs_file) hcircleq;
+
+	pthread_mutex_t mutex;
 };
 
 
