@@ -4,6 +4,10 @@
 #include <unistd.h>
 #include <sys/types.h>
 
+#if (defined(__hurd__) || defined(__gnu_hurd__)) && !defined(PATH_MAX)
+#define PATH_MAX	2048	/* XXX FIXME */
+#endif
+
 #include <gfarm/gfarm.h>
 
 #include "gfutil.h"
