@@ -33,6 +33,7 @@
 #include "gfevent.h"
 #include "hash.h"
 #include "timer.h"
+#include "thrsubr.h"
 
 #include "context.h"
 #include "liberror.h"
@@ -50,11 +51,19 @@
 #include "gfs_failover.h"
 
 #ifndef __KERNEL__
-#define SCHED_MUTEX_DCL
-#define SCHED_MUTEX_INIT(s)
-#define SCHED_MUTEX_DESTROY(s)
-#define SCHED_MUTEX_LOCK(s)
-#define SCHED_MUTEX_UNLOCK(s)
+#define SCHED_MUTEX_DCL		pthread_mutex_t sched_mutex;
+#define SCHED_MUTEX_INIT(s)	do { \
+	gfarm_mutex_init(&(s)->sched_mutex, __func__, "sched_mutex"); \
+} while (/*CONSTCOND*/0);
+#define SCHED_MUTEX_DESTROY(s)	do { \
+	gfarm_mutex_destroy(&(s)->sched_mutex, __func__, "sched_mutex"); \
+} while (/*CONSTCOND*/0);
+#define SCHED_MUTEX_LOCK(s)	do { \
+	gfarm_mutex_lock(&(s)->sched_mutex, __func__, "sched_mutex"); \
+} while (/*CONSTCOND*/0);
+#define SCHED_MUTEX_UNLOCK(s)	do { \
+	gfarm_mutex_unlock(&(s)->sched_mutex, __func__, "sched_mutex"); \
+} while (/*CONSTCOND*/0);
 #else /* __KERNEL__ */
 #define SCHED_MUTEX_DCL	pthread_mutex_t sched_mutex;
 #define SCHED_MUTEX_INIT(s)	pthread_mutex_init(&(s)->sched_mutex, NULL);
@@ -359,7 +368,7 @@ struct search_idle_host_state {
 	gfarm_off_t files, ffree, favail;
 	gfarm_int32_t bsize;
 #else
-	gfarm_off_t diskused, diskavail;
+	gfarm_uint64_t diskused, diskavail;
 #endif
 
 	gfarm_uint64_t scheduled_age;
@@ -724,8 +733,11 @@ gfarm_schedule_host_cache_purge(struct gfs_connection *gfs_server)
 	struct gfarm_hash_entry *entry;
 	struct search_idle_host_state *h;
 
-	if (staticp->search_idle_hosts_state == NULL)
+	SCHED_MUTEX_LOCK(staticp);
+	if (staticp->search_idle_hosts_state == NULL) {
+		SCHED_MUTEX_UNLOCK(staticp);
 		return (GFARM_ERR_NO_ERROR);
+	}
 
 	e = gfp_conn_hash_lookup(&staticp->search_idle_hosts_state,
 	    HOSTS_HASHTAB_SIZE,
@@ -738,11 +750,13 @@ gfarm_schedule_host_cache_purge(struct gfs_connection *gfs_server)
 		    "gfarm_schedule_host_cache_purge: "
 		    "connection is not cached: %s",
 		    gfarm_error_string(e));
+		SCHED_MUTEX_UNLOCK(staticp);
 		return (e);
 	}
 
 	h = gfarm_hash_entry_data(entry);
 	h->flags &= ~HOST_STATE_FLAG_AUTH_SUCCEED;
+	SCHED_MUTEX_UNLOCK(staticp);
 	return (GFARM_ERR_NO_ERROR);
 }
 
@@ -762,8 +776,11 @@ gfarm_schedule_host_cache_reset(struct gfm_connection *gfm_server, int nhosts,
 	    HOST_STATE_FLAG_AVAILABLE|
 	    HOST_STATE_FLAG_CACHE_WAS_USED;
 
-	if (staticp->search_idle_hosts_state == NULL)
+	SCHED_MUTEX_LOCK(staticp);
+	if (staticp->search_idle_hosts_state == NULL) {
+		SCHED_MUTEX_UNLOCK(staticp);
 		return;
+	}
 
 	for (i = 0; i < nhosts; ++i) {
 		e = gfp_conn_hash_lookup(&staticp->search_idle_hosts_state,
@@ -775,6 +792,7 @@ gfarm_schedule_host_cache_reset(struct gfm_connection *gfm_server, int nhosts,
 		h = gfarm_hash_entry_data(entry);
 		h->flags &= ~host_flags;
 	}
+	SCHED_MUTEX_UNLOCK(staticp);
 	return;
 }
 
@@ -975,7 +993,9 @@ search_idle_candidate_list_add(struct gfm_connection *gfm_server,
 void
 gfarm_schedule_search_mode_use_loadavg(void)
 {
+	SCHED_MUTEX_LOCK(staticp);
 	staticp->default_search_method = GFARM_SCHEDULE_SEARCH_BY_LOADAVG;
+	SCHED_MUTEX_UNLOCK(staticp);
 }
 
 #define IDLE_LOAD_AVERAGE	(gfarm_ctxp->schedule_idle_load * \
@@ -1194,14 +1214,24 @@ search_idle_statfs_callback(void *closure)
 	if (e == GFARM_ERR_NO_ERROR) {
 		h->flags |= HOST_STATE_FLAG_STATFS_AVAIL;
 		h->statfs_cache_time = h->rtt_cache_time;
+#if 0		/* NOTE: gfarm v2 does not use disk{used,avail} from gfsd */
 		h->diskused = blocks * bsize;
 		h->diskavail = bavail * bsize;
+		/*
+		 * If h->disk{used,avail} are updated here, gfsched -w
+		 * may include readonly hosts, because
+		 * READONLY_CONFIG_TMP_FILE is not created yet until
+		 * next heartbeat (GFS_PROTO_STATUS2).
+		 */
+#endif
+
 #ifndef __KERNEL__
 		{
 			struct gfs_ib_rdma_state *state;
 			e = gfs_ib_rdma_request_multiplexed(
-				c->state->q, c->gfs_server,
-				search_idle_rdma_callback, c, &state);
+			    c->state->q, c->gfs_server,
+			    gfarm_ctxp->schedule_rpc_timeout,
+			    search_idle_rdma_callback, c, &state);
 			if (e == GFARM_ERR_NO_ERROR) {
 				c->protocol_state = state;
 				return; /* request continues */
@@ -1248,7 +1278,9 @@ search_idle_connect_callback(void *closure)
 		} else {
 #endif
 			e = gfs_client_statfs_request_multiplexed(c->state->q,
-			    c->gfs_server, ".", search_idle_statfs_callback, c,
+			    c->gfs_server, ".",
+			    gfarm_ctxp->schedule_rpc_timeout,
+			    search_idle_statfs_callback, c,
 			    &ss);
 			if (e == GFARM_ERR_NO_ERROR) {
 				c->protocol_state = ss;
@@ -1316,6 +1348,7 @@ search_idle_load_callback(void *closure)
 			    c->h->return_value, c->h->port,
 			    gfm_client_username(s->gfm_server),
 			    &c->h->addr, s->filesystem,
+			    gfarm_ctxp->schedule_rpc_timeout,
 			    search_idle_connect_callback, c,
 			    &cs);
 			if (e == GFARM_ERR_NO_ERROR) {
@@ -1419,8 +1452,8 @@ davail_compare(const void *a, const void *b)
 	struct search_idle_host_state *const *bb = b;
 	const struct search_idle_host_state *p = *aa;
 	const struct search_idle_host_state *q = *bb;
-	const gfarm_off_t df1 = p->diskavail;
-	const gfarm_off_t df2 = q->diskavail;
+	const gfarm_uint64_t df1 = p->diskavail;
+	const gfarm_uint64_t df2 = q->diskavail;
 
 	if (df1 > df2)
 		return (-1);
@@ -1546,11 +1579,10 @@ search_idle_try_host(struct search_idle_state *s,
 	    search_idle_load_callback, c, &gls, 1);
 #else /* __KERNEL__ */
 	e = gfs_client_connect_request_multiplexed(s->q,
-			    h->return_value, h->port,
-			    gfm_client_username(s->gfm_server),
-			    &h->addr, s->filesystem,
-			    search_idle_connect_and_get_rtt_callback, c,
-			    (struct gfs_client_connect_state **)&gls);
+	    h->return_value, h->port, gfm_client_username(s->gfm_server),
+	    &h->addr, s->filesystem, gfarm_ctxp->schedule_rpc_timeout,
+	    search_idle_connect_and_get_rtt_callback, c,
+	    (struct gfs_client_connect_state **)&gls);
 #endif /* __KERNEL__ */
 	if (e != GFARM_ERR_NO_ERROR) {
 		gflog_debug(GFARM_MSG_1001446,
@@ -1990,15 +2022,18 @@ gfarm_schedule_select_host(struct gfm_connection *gfm_server,
 	if (target_domain != NULL && e == GFARM_ERR_NO_FILESYSTEM_NODE)
 		e = select_hosts(gfm_server, 1, write_mode, NULL,
 			nhosts, infos, &n, &host, &port);
-	SCHED_MUTEX_UNLOCK(staticp)
-	if (e != GFARM_ERR_NO_ERROR)
+	if (e != GFARM_ERR_NO_ERROR) {
+		SCHED_MUTEX_UNLOCK(staticp)
 		return (e);
+	}
 	if (n == 0) { /* although this shouldn't happen */
 		gflog_debug(GFARM_MSG_1001456,
 		    "gfarm_schedule_select_host: no filesystem node");
+		SCHED_MUTEX_UNLOCK(staticp)
 		return (GFARM_ERR_NO_FILESYSTEM_NODE);
 	}
 	host = strdup(host);
+	SCHED_MUTEX_UNLOCK(staticp)
 	if (host == NULL) {
 		gflog_debug(GFARM_MSG_1001457,
 		    "gfarm_schedule_select_host: no memory");
@@ -2143,16 +2178,22 @@ gfarm_schedule_host_used(const char *hostname, int port, const char *username)
 	struct gfarm_hash_entry *entry;
 	struct search_idle_host_state *h;
 
-	if (staticp->search_idle_hosts_state == NULL)
+	SCHED_MUTEX_LOCK(staticp);
+	if (staticp->search_idle_hosts_state == NULL) {
+		SCHED_MUTEX_UNLOCK(staticp);
 		return (HOST_STATE_SCHEDULED_AGE_NOT_FOUND);
+	}
 
 	e = gfp_conn_hash_lookup(&staticp->search_idle_hosts_state,
 	    HOSTS_HASHTAB_SIZE, hostname, port, username, &entry);
-	if (e != GFARM_ERR_NO_ERROR)
+	if (e != GFARM_ERR_NO_ERROR) {
+		SCHED_MUTEX_UNLOCK(staticp);
 		return (HOST_STATE_SCHEDULED_AGE_NOT_FOUND);
+	}
 
 	h = gfarm_hash_entry_data(entry);
 	h->scheduled++;
+	SCHED_MUTEX_UNLOCK(staticp);
 	return (h->scheduled_age);
 }
 
@@ -2167,18 +2208,23 @@ gfarm_schedule_host_unused(const char *hostname, int port, const char *username,
 
 	if (scheduled_age == HOST_STATE_SCHEDULED_AGE_NOT_FOUND)
 		return;
+	SCHED_MUTEX_LOCK(staticp);
 	if (staticp->search_idle_hosts_state == NULL) {
 		gflog_notice(GFARM_MSG_1004318, "%s: internal error", diag);
+		SCHED_MUTEX_UNLOCK(staticp);
 		return;
 	}
 	e = gfp_conn_hash_lookup(&staticp->search_idle_hosts_state,
 	    HOSTS_HASHTAB_SIZE, hostname, port, username, &entry);
-	if (e != GFARM_ERR_NO_ERROR)
+	if (e != GFARM_ERR_NO_ERROR) {
+		SCHED_MUTEX_UNLOCK(staticp);
 		return;
+	}
 
 	h = gfarm_hash_entry_data(entry);
 	if (h->scheduled_age == scheduled_age)
 		--h->scheduled;
+	SCHED_MUTEX_UNLOCK(staticp);
 }
 
 /* this function shouldn't belong to this file, but... */
@@ -2187,6 +2233,7 @@ gfm_host_is_in_local_net(struct gfm_connection *gfm_server, const char *host)
 {
 	gfarm_error_t e;
 	struct sockaddr addr;
+	int rv;
 
 	/* XXX it's desirable to use struct sockaddr in the scheduling cache */
 
@@ -2195,12 +2242,17 @@ gfm_host_is_in_local_net(struct gfm_connection *gfm_server, const char *host)
 	if (e != GFARM_ERR_NO_ERROR)
 		return (0);
 
+	SCHED_MUTEX_LOCK(staticp);
 	if (staticp->search_idle_local_net == NULL &&
-	    search_idle_network_list_init(gfm_server) != GFARM_ERR_NO_ERROR)
+	    search_idle_network_list_init(gfm_server) != GFARM_ERR_NO_ERROR) {
+		SCHED_MUTEX_UNLOCK(staticp);
 		return (0);
+	}
 
-	return (gfarm_hostspec_match(staticp->search_idle_local_net->network,
-	    NULL, &addr));
+	rv = gfarm_hostspec_match(staticp->search_idle_local_net->network,
+	    NULL, &addr);
+	SCHED_MUTEX_UNLOCK(staticp);
+	return (rv);
 }
 
 /*
@@ -2212,6 +2264,7 @@ gfarm_schedule_network_cache_dump(void)
 	struct search_idle_network *n;
 	char addr[GFARM_HOSTSPEC_STRLEN];
 
+	SCHED_MUTEX_LOCK(staticp);
 	for (n = staticp->search_idle_network_list; n != NULL; n = n->next) {
 		/*
 		 * the reason why we don't use inet_ntoa() here is
@@ -2226,6 +2279,7 @@ gfarm_schedule_network_cache_dump(void)
 			gflog_info(GFARM_MSG_1000175,
 			    "%s: RTT unavailable", addr);
 	}
+	SCHED_MUTEX_UNLOCK(staticp);
 }
 
 void
@@ -2242,8 +2296,10 @@ gfarm_schedule_host_cache_dump(void)
 	char disktotalbuf[GFARM_INT64STRLEN];
 	struct timeval period;
 
+	SCHED_MUTEX_LOCK(staticp);
 	if (staticp->search_idle_hosts_state == NULL) {
 		gflog_info(GFARM_MSG_1000176, "<empty>");
+		SCHED_MUTEX_UNLOCK(staticp);
 		return;
 	}
 
@@ -2322,6 +2378,7 @@ gfarm_schedule_host_cache_dump(void)
 		gflog_info(GFARM_MSG_1000177,
 		    "%s %s %s %s", hostbuf, rttbuf, loadbuf, diskbuf);
 	}
+	SCHED_MUTEX_UNLOCK(staticp);
 }
 
 void
