@@ -79,6 +79,8 @@
 #define GFMD_CONFIG		"/etc/gfmd.conf"
 #endif
 
+#define GFMD_FAILOVER_CONFIG_BASENAME	"gfmd.failover.conf"
+
 #ifndef CALLOUT_NTHREADS
 /*
  * this is number of thread pools which are used by callouts,
@@ -1228,8 +1230,135 @@ start_gfmdc_threads(void)
 		    gfarm_error_string(e));
 }
 
+static void dynamic_config_read(FILE *, const char *);
+
+static void
+dynamic_config_include_file(char *rest_of_line, const char *file, int lineno)
+{
+	gfarm_error_t e;
+	char *tmp, *s, *p, *malloced_filename = NULL;
+	FILE *config;
+	int nesting_limit = gfarm_ctxp->include_nesting_limit;
+
+	p = rest_of_line;
+	e = gfarm_config_strtoken(&p, &s);
+	if (e != GFARM_ERR_NO_ERROR) {
+		gflog_error(GFARM_MSG_UNFIXED,
+		    "file %s, line %d: parsing token (%s) failed: %s",
+		    file, lineno, p, gfarm_error_string(e));
+		return;
+	}
+	if (s == NULL) {
+		gflog_error(GFARM_MSG_UNFIXED,
+		    "file %s, line %d: missing include filename",
+		    file, lineno);
+		return;
+	}
+	e = gfarm_config_strtoken(&p, &tmp);
+	if (e != GFARM_ERR_NO_ERROR) {
+		gflog_error(GFARM_MSG_UNFIXED,
+		    "file %s, line %d: parsing token (%s) failed: %s",
+		    file, lineno, p, gfarm_error_string(e));
+		return;
+	}
+	if (tmp != NULL) {
+		gflog_error(GFARM_MSG_UNFIXED,
+		    "file %s, line %d: too many arguments for include",
+		    file, lineno);
+		return;
+	}
+
+	++gfarm_ctxp->include_nesting_level;
+	if (gfarm_ctxp->include_nesting_level > nesting_limit) {
+		gflog_error(GFARM_MSG_UNFIXED,
+		    "file %s, line %d: include nesting level %d "
+		    "exceeds include_nesting_limit %d",
+		    file, lineno,
+		    gfarm_ctxp->include_nesting_level, nesting_limit);
+		--gfarm_ctxp->include_nesting_level;
+		return;
+	}
+
+	if (s[0] != '/')  {
+		malloced_filename = gfarm_config_dirname_add(s, file);
+		if (malloced_filename == NULL) {
+			gflog_error(GFARM_MSG_UNFIXED,
+			    "file %s, line %d: no memory to include %s",
+			    file, lineno, s);
+		}
+		s = malloced_filename;
+	}
+	config = fopen(s, "r");
+	if (config == NULL) {
+		gflog_notice(GFARM_MSG_UNFIXED,
+		    "file %s, line %d: %s: cannot open include file",
+		    file, lineno, s);
+		free(malloced_filename);
+		--gfarm_ctxp->include_nesting_level;
+		return;
+	}
+	dynamic_config_read(config, s);
+	fclose(config);
+	free(malloced_filename);
+	--gfarm_ctxp->include_nesting_level;
+}
+
+/* this constant can be different from same macro in config.c */
+#define MAX_CONFIG_LINE_LENGTH	1023
+
+static void
+dynamic_config_read(FILE *config, const char *file)
+{
+	gfarm_error_t e;
+	int lineno = 0;
+	char *s, *p, buffer[MAX_CONFIG_LINE_LENGTH + 1];
+
+	while (fgets(buffer, sizeof buffer, config) != NULL) {
+		lineno++;
+		p = buffer;
+
+		e = gfarm_config_strtoken(&p, &s);
+		if (e != GFARM_ERR_NO_ERROR) {
+			gflog_error(GFARM_MSG_UNFIXED,
+			    "%s, line %d: invalid token: %s",
+			    file, lineno, gfarm_error_string(e));
+			/* don't stop, just ignore this line line */
+			continue;
+		}
+
+		if (s == NULL) /* blank or comment line */
+			continue;
+
+		if (strcmp(s, "include") == 0) {
+			dynamic_config_include_file(p, file, lineno);
+		} else {
+			/* error message will be logged by callee */
+			e = gfarm_config_apply_to_metadb(
+			    s, p, file, lineno, /* log changes */ 1);
+			(void)e;
+		}
+	}
+}
+
+static void
+dynamic_config_read_file(const char *file)
+{
+	FILE *config = fopen(file, "r");
+
+	if (config == NULL) {
+		gflog_notice(GFARM_MSG_UNFIXED,
+		    "%s: cannot open config file", file);
+		return;
+	}
+	gfarm_config_apply_begin();
+	dynamic_config_read(config, file);
+	gfarm_config_apply_end();
+	fclose(config);
+}
+
 static pthread_mutex_t transform_mutex = PTHREAD_MUTEX_INITIALIZER;
 static pthread_cond_t transform_cond = PTHREAD_COND_INITIALIZER;
+static char *failover_config_file = NULL;
 
 static void
 transform_to_master(void)
@@ -1287,6 +1416,7 @@ transform_to_master(void)
 	start_gfmdc_threads();
 
 	giant_lock();
+	dynamic_config_read_file(failover_config_file);
 	mdhost_set_self_as_default_master();
 	giant_unlock();
 
@@ -1644,6 +1774,7 @@ gfmd_modules_init_default(int table_size)
 	host_init();
 	user_init();
 	group_init();
+	quota_init();
 
 	/* filesystem */
 	inode_init();
@@ -1653,7 +1784,6 @@ gfmd_modules_init_default(int table_size)
 	if (gfarm_backend_db_type == GFARM_BACKEND_DB_TYPE_NONE)
 		xattr_init_cache_all();
 	xattr_init();
-	quota_init();
 	dirset_init();
 	quota_dir_init();
 
@@ -1707,7 +1837,7 @@ main(int argc, char **argv)
 		program_name = basename(argv[0]);
 	gflog_set_identifier(program_name);
 
-	while ((ch = getopt(argc, argv, "F:L:P:df:p:Ss:tv")) != -1) {
+	while ((ch = getopt(argc, argv, "F:L:P:df:p:Ss:tu:v")) != -1) {
 		switch (ch) {
 		case 'F':
 			syslog_file = optarg;
@@ -1747,6 +1877,9 @@ main(int argc, char **argv)
 		case 't':
 			file_trace = 1;
 			break;
+		case 'u':
+			failover_config_file = optarg;
+			break;
 		case 'v':
 			gflog_auth_set_verbose(1);
 			break;
@@ -1758,6 +1891,17 @@ main(int argc, char **argv)
 
 	if (config_file == NULL)
 		config_file = GFMD_CONFIG;
+	if (failover_config_file == NULL)
+		failover_config_file = GFMD_FAILOVER_CONFIG_BASENAME;
+	if (failover_config_file[0] != '/')  {
+		failover_config_file = gfarm_config_dirname_add(
+		    failover_config_file, config_file);
+		if (failover_config_file == NULL) {
+			fprintf(stderr, "gfmd: no memory for %s\n",
+			    GFMD_FAILOVER_CONFIG_BASENAME);
+			exit(1);
+		}
+	}
 	gfarm_metadb_version_major = gfarm_version_major();
 	gfarm_metadb_version_minor = gfarm_version_minor();
 	gfarm_metadb_version_teeny = gfarm_version_teeny();
@@ -1930,9 +2074,12 @@ main(int argc, char **argv)
 			    "another master gfmd is already running");
 		}
 		/* these functions write db, thus, must be after db_thread  */
-		inode_remove_orphan(); /* should be before
-					  inode_check_and_repair() */
-		inode_check_and_repair();
+		if (!gfarm_read_only_mode()) {
+			/* do this before inode_check_and_repair() */
+			inode_remove_orphan();
+
+			inode_check_and_repair();
+		}
 	}
 	inode_free_orphan();
 	gflog_info(GFARM_MSG_1004204, "end bootstrap");
@@ -1953,9 +2100,9 @@ main(int argc, char **argv)
 
 	/* master */
 
-	failover_notify();
 	quota_check_init();
 	replica_check_init();
+	failover_notify();
 	accepting_loop(sock);
 
 	/*NOTREACHED*/
