@@ -805,7 +805,7 @@ host_is_readonly_update(struct host *h)
 	static const char diag[] = "host_is_readonly_update";
 
 	abstract_host_mutex_lock(&h->ah, diag);
-	h->is_readonly = host_info_is_readonly(&h->hi);
+	h->is_readonly = gfarm_host_info_is_readonly(&h->hi);
 	abstract_host_mutex_unlock(&h->ah, diag);
 }
 
@@ -1238,7 +1238,7 @@ host_new(struct gfarm_host_info *hi, struct callout *callout)
 #endif
 	h->status_reply_waiting = 0;
 	h->report_flags = 0;
-	h->is_readonly = host_info_is_readonly(hi);
+	h->is_readonly = gfarm_host_info_is_readonly(hi);
 	h->status.loadavg_1min =
 	h->status.loadavg_5min =
 	h->status.loadavg_15min = 0.0;
@@ -1887,6 +1887,10 @@ gfm_server_host_info_set(struct peer *peer, int from_client, int skip)
 		e = GFARM_ERR_ALREADY_EXISTS;
 	} else if ((e = host_info_verify(&hi, diag)) != GFARM_ERR_NO_ERROR) {
 		/* nothing to do */
+	} else if (gfarm_read_only_mode()) {
+		gflog_debug(GFARM_MSG_UNFIXED, "%s (%s@%s) during read_only",
+		    diag, peer_get_username(peer), peer_get_hostname(peer));
+		e = GFARM_ERR_READ_ONLY_FILE_SYSTEM;
 	} else if ((e = host_enter(&hi, NULL)) != GFARM_ERR_NO_ERROR) {
 		/* nothing to do */
 	} else if ((e = db_host_add(&hi)) != GFARM_ERR_NO_ERROR) {
@@ -1992,6 +1996,11 @@ gfm_server_host_info_modify(struct peer *peer, int from_client, int skip)
 		needs_free = 1;
 	} else if ((e = host_info_verify(&hi, diag)) != GFARM_ERR_NO_ERROR) {
 		needs_free = 1;
+	} else if (gfarm_read_only_mode()) {
+		gflog_debug(GFARM_MSG_UNFIXED, "%s (%s@%s) during read_only",
+		    diag, peer_get_username(peer), peer_get_hostname(peer));
+		e = GFARM_ERR_READ_ONLY_FILE_SYSTEM;
+		needs_free = 1;
 	} else if ((e = db_host_modify(&hi,
 	    DB_HOST_MOD_ARCHITECTURE|DB_HOST_MOD_NCPU|DB_HOST_MOD_FLAGS,
 	    /* XXX */ 0, NULL, 0, NULL)) != GFARM_ERR_NO_ERROR) {
@@ -2093,6 +2102,10 @@ gfm_server_host_info_remove(struct peer *peer, int from_client, int skip)
 		gflog_debug(GFARM_MSG_1001572,
 			"operation is not permitted");
 		e = GFARM_ERR_OPERATION_NOT_PERMITTED;
+	} else if (gfarm_read_only_mode()) {
+		gflog_debug(GFARM_MSG_UNFIXED, "%s (%s@%s) during read_only",
+		    diag, peer_get_username(peer), peer_get_hostname(peer));
+		e = GFARM_ERR_READ_ONLY_FILE_SYSTEM;
 	} else
 		e = host_info_remove(hostname, diag);
 	free(hostname);
