@@ -1338,6 +1338,9 @@ mdhost_db_modify_default_master(struct mdhost *mh,
 	return (e);
 }
 
+/* needs giant_lock */
+static int master_transformation_has_to_be_propagated = 0;
+
 /* PREREQUISITE: giant_lock */
 void
 mdhost_set_self_as_default_master(void)
@@ -1347,6 +1350,15 @@ mdhost_set_self_as_default_master(void)
 	static const char diag[] = "mdhost_set_self_as_default_master";
 
 	mdhost_set_is_default_master(self, 1);
+
+	if (gfarm_read_only_mode()) {
+		master_transformation_has_to_be_propagated = 1;
+		gflog_notice(GFARM_MSG_UNFIXED,
+		    "propagating master transformation is postponed "
+		    "due to read_only");
+		return;
+	}
+
 	if ((e = mdhost_db_modify_default_master(self, &self->ms, diag)) !=
 	    GFARM_ERR_NO_ERROR)
 		;
@@ -1355,6 +1367,18 @@ mdhost_set_self_as_default_master(void)
 		    diag, gfarm_error_string(e));
 }
 
+/* PREREQUISITE: giant_lock */
+void
+mdhost_read_only_disabled(void)
+{
+	if (master_transformation_has_to_be_propagated) {
+		master_transformation_has_to_be_propagated = 0;
+		mdhost_set_self_as_default_master();
+		gflog_notice(GFARM_MSG_UNFIXED,
+		    "master transformation is propagated");
+	}
+}
+
 static gfarm_error_t
 metadb_server_check_write_access(struct peer *peer, int from_client,
 	const char *diag)
@@ -1405,6 +1429,12 @@ gfm_server_metadb_server_set(struct peer *peer, int from_client, int skip)
 	} else if ((e = metadb_server_verify(&ms, diag))
 	    != GFARM_ERR_NO_ERROR) {
 		/* nothing to do */
+	} else if (gfarm_read_only_mode()) {
+		gflog_debug(GFARM_MSG_UNFIXED, "%s (%s@%s) for "
+		    "name %s port %d flags 0x%x during read_only",
+		    diag, peer_get_username(peer), peer_get_hostname(peer),
+		    ms.name, (int)ms.port, (int)ms.flags);
+		e = GFARM_ERR_READ_ONLY_FILE_SYSTEM;
 	} else if ((e = mdhost_enter_internal(&ms, &mh))
 	    != GFARM_ERR_NO_ERROR) {
 		/* nothing to do */
@@ -1496,6 +1526,12 @@ gfm_server_metadb_server_modify(struct peer *peer, int from_client, int skip)
 	} else if ((e = metadb_server_verify(&ms, diag))
 		   != GFARM_ERR_NO_ERROR) {
 		/* nothing to do */
+	} else if (gfarm_read_only_mode()) {
+		gflog_debug(GFARM_MSG_UNFIXED, "%s (%s@%s) for "
+		    "name %s port %d flags 0x%x during read_only",
+		    diag, peer_get_username(peer), peer_get_hostname(peer),
+		    ms.name, (int)ms.port, (int)ms.flags);
+		e = GFARM_ERR_READ_ONLY_FILE_SYSTEM;
 	} else {
 		mdhost_modify_in_cache_internal(mh, &ms);
 		if (isdm) {
@@ -1554,6 +1590,12 @@ gfm_server_metadb_server_remove(struct peer *peer, int from_client, int skip)
 		e = GFARM_ERR_OPERATION_NOT_PERMITTED;
 		gflog_debug(GFARM_MSG_1002967,
 		    "%s: cannot remove self host", diag);
+	} else if (gfarm_read_only_mode()) {
+		gflog_debug(GFARM_MSG_UNFIXED, "%s (%s@%s) for "
+		    "name %s during read_only",
+		    diag, peer_get_username(peer), peer_get_hostname(peer),
+		    name);
+		e = GFARM_ERR_READ_ONLY_FILE_SYSTEM;
 	} else if ((e = mdhost_remove_in_cache(name)) != GFARM_ERR_NO_ERROR) {
 		gflog_debug(GFARM_MSG_1002968,
 		    "%s: mdhost_remove_in_cache(%s) failed: %s",
