@@ -176,6 +176,15 @@ dirquota_softlimit_exceed(struct quota_metadata *q, struct dirset *ds)
 	}
 }
 
+static void
+quota_info_usage_clear(struct gfarm_quota_info *usage)
+{
+	usage->space = 0;
+	usage->num = 0;
+	usage->phy_space = 0;
+	usage->phy_num = 0;
+}
+
 static void
 quota_usage_clear(struct gfarm_quota_subject_info *usage)
 {
@@ -200,17 +209,34 @@ usage_tmp_clear_group(void *closure, struct group *g)
 static void
 usage_tmp_clear(void)
 {
-	user_foreach(NULL, usage_tmp_clear_user, 0);
-	group_foreach(NULL, usage_tmp_clear_group, 0);
+	user_foreach(NULL, usage_tmp_clear_user,
+	    USER_FOREARCH_FLAG_INCLUDING_INVALID);
+	group_foreach(NULL, usage_tmp_clear_group,
+	    GROUP_FOREARCH_FLAG_INCLUDING_INVALID);
 }
 
 static void
-usage_to_quota(struct gfarm_quota_subject_info *src_usage, struct quota *dst)
+usage_to_quota(struct gfarm_quota_subject_info *src_usage, struct quota *dst,
+	const char *type, const char *name)
 {
-	dst->space = src_usage->space;
-	dst->num = src_usage->num;
-	dst->phy_space = src_usage->phy_space;
-	dst->phy_num = src_usage->phy_num;
+	if (dst->space != src_usage->space ||
+	    dst->num != src_usage->num ||
+	    dst->phy_space != src_usage->phy_space ||
+	    dst->phy_num != src_usage->phy_num) {
+		gflog_warning(GFARM_MSG_UNFIXED,
+		    "%s %s: unexpected quota update: "
+		    "space:%llu->%llu, num:%llu->%llu, "
+		    "phy_space:%llu->%llu, phy_num:%llu->%llu",
+		    type, name,
+		    (long long)dst->space, (long long)src_usage->space,
+		    (long long)dst->num, (long long)src_usage->num,
+		    (long long)dst->phy_space, (long long)src_usage->phy_space,
+		    (long long)dst->phy_num, (long long)src_usage->phy_num);
+		dst->space = src_usage->space;
+		dst->num = src_usage->num;
+		dst->phy_space = src_usage->phy_space;
+		dst->phy_num = src_usage->phy_num;
+	}
 }
 
 static void
@@ -219,9 +245,10 @@ quota_update_usage_user(void *closure, struct user *u)
 	struct quota *q = user_quota(u);
 	struct gfarm_quota_subject_info *usage_tmp = user_usage_tmp(u);
 
-	usage_to_quota(usage_tmp, q);
+	usage_to_quota(usage_tmp, q, "user", user_name(u));
 	quota_softlimit_exceed_user(q, u);
 
+#if 0 /* this is OK since gfarm-2.7.17 */
 	if (!user_is_valid(u))
 		gflog_notice(GFARM_MSG_1004294,
 		    "quota_check: removed user(%s), Usage: "
@@ -229,6 +256,7 @@ quota_update_usage_user(void *closure, struct user *u)
 		    user_name_with_invalid(u),
 		    (long long)q->space, (long long)q->num,
 		    (long long)q->phy_space, (long long)q->phy_num);
+#endif
 }
 
 static void
@@ -237,9 +265,10 @@ quota_update_usage_group(void *closure, struct group *g)
 	struct quota *q = group_quota(g);
 	struct gfarm_quota_subject_info *usage_tmp = group_usage_tmp(g);
 
-	usage_to_quota(usage_tmp, q);
+	usage_to_quota(usage_tmp, q, "group", group_name(g));
 	quota_softlimit_exceed_group(q, g);
 
+#if 0 /* this is OK since gfarm-2.7.17 */
 	if (!group_is_valid(g))
 		gflog_notice(GFARM_MSG_1004295,
 		    "quota_check: removed group(%s), Usage: "
@@ -247,13 +276,16 @@ quota_update_usage_group(void *closure, struct group *g)
 		    group_name_with_invalid(g),
 		    (long long)q->space, (long long)q->num,
 		    (long long)q->phy_space, (long long)q->phy_num);
+#endif
 }
 
 static void
 quota_update_usage(void)
 {
-	user_foreach(NULL, quota_update_usage_user, 0);
-	group_foreach(NULL, quota_update_usage_group, 0);
+	user_foreach(NULL, quota_update_usage_user,
+	    USER_FOREARCH_FLAG_INCLUDING_INVALID);
+	group_foreach(NULL, quota_update_usage_group,
+	    GROUP_FOREARCH_FLAG_INCLUDING_INVALID);
 }
 
 static gfarm_time_t
@@ -382,6 +414,8 @@ quota_user_set_one_from_db(void *closure, struct gfarm_quota_info *qi)
 		quota_user_remove_db(qi->name);
 	} else {
 		struct quota *q = user_quota(u);
+
+		quota_info_usage_clear(qi); /* usage in DB cannot be trusted */
 		quota_convert_2(qi, q);
 		q->on_db = 1; /* load from db */
 	}
@@ -400,6 +434,8 @@ quota_group_set_one_from_db(void *closure, struct gfarm_quota_info *qi)
 		quota_group_remove_db(qi->name);
 	} else {
 		struct quota *q = group_quota(g);
+
+		quota_info_usage_clear(qi); /* usage in DB cannot be trusted */
 		quota_convert_2(qi, q);
 		q->on_db = 1; /* load from db */
 	}
@@ -433,7 +469,7 @@ quota_data_init(struct quota *q)
 	/* disable all softlimit */
 	q->grace_period = GFARM_QUOTA_INVALID;
 	/* gfarmadm do not execute gfquotacheck yet */
-	q->space = QUOTA_NOT_CHECK_YET;
+	q->space = 0;
 	q->space_exceed = GFARM_QUOTA_INVALID;
 	q->space_soft = GFARM_QUOTA_INVALID;
 	q->space_hard = GFARM_QUOTA_INVALID;
@@ -1090,7 +1126,6 @@ quota_user_remove(struct user *u)
 		quota_user_remove_db(user_name(u));
 		q->on_db = 0;
 	}
-	q->space = QUOTA_NOT_CHECK_YET;
 }
 
 void
@@ -1102,7 +1137,6 @@ quota_group_remove(struct group *g)
 		quota_group_remove_db(group_name(g));
 		q->on_db = 0;
 	}
-	q->space = QUOTA_NOT_CHECK_YET;
 }
 
 /*
@@ -1191,18 +1225,6 @@ quota_check_thread(void *arg)
 	return (NULL);
 }
 
-static gfarm_error_t
-quota_check_at_the_start(struct quota_check_control *ctl)
-{
-	gfarm_error_t e = create_detached_thread(quota_check_thread, ctl);
-
-	if (e != GFARM_ERR_NO_ERROR)
-		return (e);
-
-	quota_check_start(ctl);
-	quota_check_wait_for_end(ctl);
-	return (GFARM_ERR_NO_ERROR);
-}
 
 /*
  * quota_check
@@ -1605,17 +1627,21 @@ quota_check_init(void)
 {
 	gfarm_error_t e;
 
-	e = quota_check_at_the_start(&quota_check_ctl);
+	e = create_detached_thread(quota_check_thread, &quota_check_ctl);
 	if (e != GFARM_ERR_NO_ERROR)
 		gflog_fatal(GFARM_MSG_1004299,
 		    "create_detached_thread(quota_check): %s",
 		    gfarm_error_string(e));
+	/* quota_check_start() at startup is unnecessary since gfarm-2.7.17 */
 
-	e = quota_check_at_the_start(&dirquota_check_ctl);
+	e = create_detached_thread(quota_check_thread, &dirquota_check_ctl);
 	if (e != GFARM_ERR_NO_ERROR)
 		gflog_fatal(GFARM_MSG_1004641,
 		    "create_detached_thread(dirquota_check): %s",
 		    gfarm_error_string(e));
+	/* quota_check_start() for dirquota at startup is still necessary */
+	quota_check_start(&dirquota_check_ctl);
+	/* don't call quota_check_wait_for_end() here to make startup faster */
 }
 
 /*
@@ -1663,7 +1689,8 @@ quota_get_common(struct peer *peer, int from_client, int skip, int is_group)
 	if (is_group) {
 		if (strcmp(name, "") == 0)
 			e = GFARM_ERR_NO_SUCH_GROUP;
-		else if ((group = group_lookup(name)) == NULL) {
+		else if ((group = group_lookup_including_invalid(name))
+		    == NULL) {
 			if (user_is_admin(peer_user))
 				e = GFARM_ERR_NO_SUCH_GROUP;
 			else  /* hidden groupnames */
@@ -1683,7 +1710,7 @@ quota_get_common(struct peer *peer, int from_client, int skip, int is_group)
 			user = peer_user; /* permit not-admin */
 		else if (!user_is_admin(peer_user))
 			e = GFARM_ERR_OPERATION_NOT_PERMITTED;
-		else if ((user = user_lookup(name)) == NULL)
+		else if ((user = user_lookup_including_invalid(name)) == NULL)
 			e = GFARM_ERR_NO_SUCH_USER;
 	}
 	if (e != GFARM_ERR_NO_ERROR) {
@@ -1822,6 +1849,13 @@ quota_set_common(struct peer *peer, int from_client, int skip, int is_group)
 	} else if ((e = quota_lookup(qi.name, is_group, &q, diag))
 	    != GFARM_ERR_NO_ERROR) {
 		goto end;
+	} else if (gfarm_read_only_mode()) {
+		e = GFARM_ERR_READ_ONLY_FILE_SYSTEM;
+		gflog_debug(GFARM_MSG_UNFIXED, "%s (%s@%s) for "
+		    "name %s during read_only",
+		    diag, peer_get_username(peer), peer_get_hostname(peer),
+		    qi.name);
+		goto end;
 	}
 
 	/* set limits */
