@@ -255,6 +255,9 @@ user_remove_internal(const char *username, int update_quota)
 	/* free group assignment */
 	while ((ga = u->groups.group_next) != &u->groups)
 		grpassign_remove(ga);
+
+	/* NOTE: this user remains in struct inode, dirset and quota_dir */
+
 	/*
 	 * do not purge the hash entry.  Instead, invalidate it so
 	 * that it can be activated later.
@@ -324,6 +327,12 @@ user_name(struct user *u)
 	    u->ui.username : REMOVED_USER_NAME);
 }
 
+char *
+user_name_even_invalid(struct user *u)
+{
+	return (u != NULL ? u->ui.username : REMOVED_USER_NAME);
+}
+
 char *
 user_name_with_invalid(struct user *u)
 {
@@ -406,10 +415,11 @@ user_get_dirsets(struct user *u)
 
 void
 user_foreach(void *closure, void (*callback)(void *, struct user *),
-	int valid_only)
+	int flags)
 {
 	struct gfarm_hash_iterator it;
 	struct user **u;
+	int valid_only = (flags & USER_FOREARCH_FLAG_VALID_ONLY) != 0;
 
 	for (gfarm_hash_iterator_begin(user_hashtab, &it);
 	     !gfarm_hash_iterator_is_end(&it);
@@ -916,6 +926,10 @@ gfm_server_user_info_set(struct peer *peer, int from_client, int skip)
 			"User already exists");
 	} else if ((e = user_info_verify(&ui, diag)) != GFARM_ERR_NO_ERROR) {
 		/* nothing to do */
+	} else if (gfarm_read_only_mode()) {
+		gflog_debug(GFARM_MSG_UNFIXED, "%s (%s@%s) during read_only",
+		    diag, peer_get_username(peer), peer_get_hostname(peer));
+		e = GFARM_ERR_READ_ONLY_FILE_SYSTEM;
 	} else {
 		e = user_enter(&ui, NULL);
 		if (e == GFARM_ERR_NO_ERROR) {
@@ -1023,6 +1037,10 @@ gfm_server_user_info_modify(struct peer *peer, int from_client, int skip)
 	} else if ((e = user_info_verify(&ui, diag)) != GFARM_ERR_NO_ERROR) {
 		gflog_debug(GFARM_MSG_1003462,
 		    "%s: user_info_verify: %s", diag, gfarm_error_string(e));
+	} else if (gfarm_read_only_mode()) {
+		gflog_debug(GFARM_MSG_UNFIXED, "%s (%s@%s) during read_only",
+		    diag, peer_get_username(peer), peer_get_hostname(peer));
+		e = GFARM_ERR_READ_ONLY_FILE_SYSTEM;
 	} else if ((e = user_modify(u, &ui)) != GFARM_ERR_NO_ERROR) {
 		gflog_debug(GFARM_MSG_1003463,
 		    "%s: user_modify: %s", diag, gfarm_error_string(e));
@@ -1088,6 +1106,10 @@ gfm_server_user_info_remove(struct peer *peer, int from_client, int skip)
 		gflog_debug(GFARM_MSG_1001513,
 			"operation is not permitted");
 		e = GFARM_ERR_OPERATION_NOT_PERMITTED;
+	} else if (gfarm_read_only_mode()) {
+		gflog_debug(GFARM_MSG_UNFIXED, "%s (%s@%s) during read_only",
+		    diag, peer_get_username(peer), peer_get_hostname(peer));
+		e = GFARM_ERR_READ_ONLY_FILE_SYSTEM;
 	} else
 		e = user_info_remove(username, diag);
 	free(username);
